/*****************************************************************************
*
* Emrah Akin Sisbot
* August 10
*
* File    : sparkStruct.h
* Brief   : Data structures for Spark module
*
******************************************************************************/

#ifndef SPARK_STRUCT_H
#define SPARK_STRUCT_H

/*----------------------------------------------------------------------------
  INCLUDES
-----------------------------------------------------------------------------*/

#include <genBasic/genBasicStruct.h>

#include "sparkConst.h"

/*----------------------------------------------------------------------------
  STRUCTURES
-----------------------------------------------------------------------------*/
/* DON'T FORGET: All fields in the STR of the module should have fix lengths         */
/*               Thus use of dynamic memory fields, and pointers in STR is forbidden */

typedef struct STRUCT_SPARK_P3D {
  GEN_STRING128 p3dFileName;
  int enableGraphic;
  int enableMight;
} SPARK_P3D;

typedef struct STRUCT_SPARK_ENTITY_STATE {
  GEN_STRING128 name;
  GEN_BOOL state;
} SPARK_ENTITY_STATE;

typedef struct STRUCT_SPARK_SCE {
  GEN_STRING128 name;
} SPARK_SCE;

typedef struct STRUCT_SPARK_STATE {
  GEN_STRING128 savePath; //Not used anymore
  int posPosterAv;
  int armConfPosterAv;
  int handPosterAv;
  int ptPosterAv;
  int robotConfPosterAv;
  int vimanPosterAv;
  int sparkObjPosPosterAv;
  int useObjInsideSparkEnabled;
  int hrp2ConfPosterAv;
  int mocapPosterAv;
  int humanConfPosterAv;
  int bodyPartsPosterAv;
  int oroConnectionAv;
  int oroPrintEnabled;
  int vimanPrintEnabled;
  int mightabilitiesEnabled;
  int robotHandPosterAv;
  int niutPosterAv;
  int kinectPosterAv;
  int robotFingersPosterAv;
#ifdef PR2
  int pr2ConfigPostersAv; //are Pose and Joint state posters for PR2 available?
#endif
} SPARK_STATE;

typedef struct STRUCT_SPARK_SAVE_SCENARIO {
  GEN_STRING128 savePath;
  GEN_STRING128 fileName;
} SPARK_SAVE_SCENARIO;


//-------------------- TOOLS ---------------------------

typedef struct STRUCT_SPARK_6D_COORD {
  double x;
  double y;
  double z;
  double yaw;
  double pitch;
  double roll;
} SPARK_6D_COORD;

typedef struct STRUCT_SPARK_3D_COORD {
  double x;
  double y;
  double z;
} SPARK_3D_COORD;

//-------------------- INTERFACE ------------------------

typedef struct STRUCT_SPARK_CAM_POS {
  double xdest;
  double ydest;
  double zdest;
  double dist;
  double hrot;
  double vrot;
} SPARK_CAM_POS;

typedef struct STRUCT_SPARK_INTERFACE_PARAMS {
  int width;
  int height;
  GEN_BOOL saveInterface;
  GEN_BOOL floor;
  GEN_BOOL walls;
  GEN_BOOL shadows;
  GEN_BOOL tiles;
  GEN_BOOL antialiasing;
} SPARK_INTERFACE_PARAMS;

typedef struct STRUCT_SPARK_INTERFACE_AGENT_PARAMS {
  GEN_STRING64 agentName;
  GEN_BOOL showFov;
  GEN_BOOL showPoiting;
  GEN_BOOL showVisibleObjects;
} SPARK_INTERFACE_AGENT_PARAMS;

typedef struct STRUCT_SPARK_INTERFACE_MIGHTABILITY_PARAMS {
  int showVisibleObjects;
  int showReachableObjects;
} SPARK_INTERFACE_MIGHTABILITY_PARAMS;


typedef enum ENUM_SPARK_INTERFACE_CHANGE_OPTION {
  BB = 0,
  GHOST = 1
} SPARK_INTERFACE_CHANGE_OPTION;

typedef struct STRUCT_SPARK_CHANGE_DISPLAY_PARAMS {
  SPARK_INTERFACE_CHANGE_OPTION interface_change_option; 
  GEN_BOOL is_option_true;
} SPARK_CHANGE_DISPLAY_PARAMS;

//-------------------- ENVIRONMENT -----------------------

typedef struct STRUCT_SPARK_OBJECT_PARAMS {
  double MHP_OBJECT_UPDATE_DIST;
  double MHP_OBJECT_UPDATE_ANGLE;
} SPARK_OBJECT_PARAMS;

typedef struct STRUCT_SPARK_M3D_OBJECT {
  GEN_STRING64 name;
  SPARK_6D_COORD coords;
} SPARK_M3D_OBJECT;

typedef struct STRUCT_SPARK_M3D_OBJECT_SIMU_PARAMS {
  GEN_STRING64 name;
  SPARK_6D_COORD coords;
  int hasDisappeared;
} SPARK_M3D_OBJECT_SIMU_PARAMS;

typedef struct STRUCT_SPARK_M3D_OBJECT_SIMU {
  GEN_STRING64 name;
  SPARK_6D_COORD coords;
  int hasDisappeared;
  int isnew;
} SPARK_M3D_OBJECT_SIMU;

typedef enum ENUM_SPARK_OBJECT_MOTION {
  OBJ_NONE = 0,
  OBJ_STATIC = 1,
  OBJ_MOVING = 2
} SPARK_OBJECT_MOTION;

typedef struct STRUCT_SPARK_ROBOT_POSTERS {
  GEN_STRING64 armConfPostername;
  GEN_STRING64 handConfPostername;
  GEN_STRING64 posPostername; 
  GEN_STRING64 ptPostername;
  GEN_STRING64 confPosterName;
#ifdef PR2
  GEN_STRING64 pr2PosePosterName;
  GEN_STRING64 pr2JointStatePosterName;
  GEN_STRING64 pr2JointMapPosterName;
#endif
} SPARK_ROBOT_POSTERS;

typedef enum STRUCT_SPARK_POSTER_TYPE_ENUM {
  USE_MOCAP  = 0,
  USE_GEST   = 1,
  USE_HUMPOS = 2,
  USE_CONF  = 3,
  USE_BODY_PARTS = 4,
  USE_VIMAN = 5,
  USE_OBJ_POS = 6,
  USE_NIUT = 7,
  USE_OBJ_INSIDE_SPARK = 8
} SPARK_POSTER_TYPE_ENUM;

typedef enum STRUCT_SPARK_HUMAN_STATE_ENUM {
  // must be the same as BT_... constants in Move3d hri_planner
  STANDING = 0,
  SITTING = 1,
  MOVING = 2,
  STANDING_TRANSPARENT = 3 // For optimistic planning
} SPARK_HUMAN_STATE_ENUM;

typedef struct STRUCT_SPARK_AGENT_POS {
  GEN_STRING64 name;
  SPARK_3D_COORD pos;
  double pan;
  double tilt;
  SPARK_HUMAN_STATE_ENUM state;
  int nouse;
} SPARK_AGENT_POS;

typedef struct STRUCT_SPARK_JOINT_ABS_POS_PARAM {
    // Joint AbsPos
    GEN_STRING128 robotName;
    GEN_STRING128 jointName;
} SPARK_JOINT_ABS_POS_PARAM;

typedef struct STRUCT_SPARK_CONFIGURATION {
  int changed;
  int dofNb;
  double dof[60];
} SPARK_CONFIGURATION;

/* nx  ox  ax  pz
   ny  oy  ay  py
   nz  oz  az  pz  */
typedef struct STRUCT_SPARK_POS_MATRIX {
  double nx, ny, nz;
  double ox, oy, oz;
  double ax, ay, az;
  double px, py, pz;
} SPARK_POS_MATRIX;

typedef struct STRUCT_SPARK_OBJECT_POSITION {
  unsigned long tacq_usec;	/** time of acquisition (microseconds) of the left image */
  unsigned long tacq_sec;	/** time of acquisition (seconds)  of the left image*/
  GEN_STRING64 name;	/* object name */
  int visible;
  SPARK_OBJECT_MOTION motion;
  SPARK_POS_MATRIX absPosMatrix;
} SPARK_OBJECT_POSITION;

typedef struct STRUCT_SPARK_OBJECT_POSITION_ARRAY {
  int nbObjects;
  int unused;
  SPARK_OBJECT_POSITION objects[SPARK_MAX_THINGS_NB];
} SPARK_OBJECT_POSITION_ARRAY;

typedef struct STRUCT_SPARK_INPUT_POSTER {
  SPARK_POSTER_TYPE_ENUM posterType;
  int unused;
  GEN_STRING64 name;
} SPARK_INPUT_POSTER;

typedef struct STRUCT_M3D_ROBOT {
  GEN_STRING64 name;
  double q[M3D_MAX_DOF];
  int length;
  int unused;
} M3D_ROBOT;

typedef struct STRUCT_M3D_FREEFLYER {
  GEN_STRING64 name;
  double q[6];
} M3D_FREEFLYER;

typedef struct STRUCT_SPARK_BOUNDING_BOX {
  GEN_STRING64 name;
  double x;
  double y;
  double z;
} SPARK_BOUNDING_BOX;

typedef struct STRUCT_SPARK_HUMAN_BODY_PARTS {
  GEN_STRING64 name;
  int headAv;
  int torsoAv;
  int leftHandAv;
  int rightHandAv;
  int gazeAv;
  int available;
  SPARK_6D_COORD head;
  SPARK_6D_COORD torso;
  SPARK_6D_COORD leftHand;
  SPARK_6D_COORD rightHand;
  SPARK_6D_COORD gaze;
} SPARK_HUMAN_BODY_PARTS;

typedef struct STRUCT_SPARK_HUMAN_BODY_PARTS_LIST {
  SPARK_HUMAN_BODY_PARTS human[SPARK_MAX_AGENT_NB];
  int humanNb;
  int unused;
} SPARK_HUMAN_BODY_PARTS_LIST;

typedef struct STRUCT_SPARK_DISTANCE {
  double minDistRobToHum;
  double costDistRobToHum;
} SPARK_DISTANCE;

/** Inferrence Management in sparkEnvironment

/** different type of inferrences managed*/
typedef enum ENUM_SPARK_INFERRENCE_TYPE {
  SPARK_NO_INFERRENCE =  0,
  SPARK_PRECISE_ROBOT_HAND = 1,
  SPARK_ROUGH_ROBOT_HAND = 2,
  SPARK_HUMAN_HAND =  3,
  SPARK_IS_IN = 4
} SPARK_INFERRENCE_TYPE;

/** different type of inferrences managed*/
typedef enum ENUM_SPARK_INFERRENCE_VALIDITY {
  SPARK_NO_PROBABILITY =  0,
  SPARK_HIGHLY_PROBABLE = 1,
  SPARK_PROBABLE = 2,
  SPARK_AVERAGE =  3,
  SPARK_UNPROBABLE = 4,
  SPARK_HIGHLY_UNPROBABLE =  5
} SPARK_INFERRENCE_VALIDITY;

typedef struct STRUCT_SPARK_SET_INFERRENCE_FOR_OBJECT_PARAMS {
  GEN_STRING64 objectName;
  int hasInferrence;
  GEN_STRING64 agentOrObjectName;
  int agentPartNum;
  SPARK_INFERRENCE_TYPE inferrenceType;
  double perceptionInferrenceConflictThresholdMultiply; //
} SPARK_SET_INFERRENCE_FOR_OBJECT_PARAMS;

typedef struct STRUCT_SPARK_SET_GRASPED_OBJECT_PARAMS {
  GEN_STRING64 name;
  int is_grasping_object;
  int grasped_object_arm_id;
} SPARK_SET_GRASPED_OBJECT_PARAMS;

typedef struct STRUCT_SPARK_SET_OBJECT_TO_GRIPPER_POSE_PARAMS {
  GEN_STRING64 name;
  int arm_id;
  double x;
  double y;
  double z;
  double rx;
  double ry;
  double rz;
} SPARK_SET_OBJECT_TO_GRIPPER_POSE_PARAMS;

typedef struct STRUCT_SPARK_KINECT_FRAME {
  GEN_BOOL enable;
  double x;
  double y;
  double z;
  double rx;
  double ry;
  double rz;
} SPARK_KINECT_FRAME;


//---------------- SITUATION ASSESSMENT ------------------

typedef enum ENUM_SPARK_PLACEMENT_RELATION {
  ISIN     = 0,
  ISON     = 1,
  ISNEXTTO = 2,
  NOPLR    = 3
} SPARK_PLACEMENT_RELATION;

typedef enum ENUM_SPARK_VISIBILITY {
  INVISIBLE = 0,
  VISIBLE   = 1
} SPARK_VISIBILITY;

typedef enum ENUM_SPARK_VISIBILITY_PLACEMENT {
  NO_VP = 0,
  OOF = 3,
  FOV = 2,
  FOA = 1
} SPARK_VISIBILITY_PLACEMENT;

typedef enum ENUM_SPARK_SPATIAL_RELATION {
  NO_RELATION = 0,
  NEAR_FRONT = 1,
  NEAR_FRONT_LEFT =  2,
  NEAR_LEFT = 3,
  NEAR_BACK_LEFT = 4,
  NEAR_BACK = 5,
  NEAR_BACK_RIGHT = 6,
  NEAR_RIGHT = 7,
  NEAR_FRONT_RIGHT = 8,
  FAR_FRONT = 9,
  FAR_FRONT_LEFT =  10,
  FAR_LEFT = 11,
  FAR_BACK_LEFT = 12,
  FAR_BACK = 13,
  FAR_BACK_RIGHT = 14,
  FAR_RIGHT = 15,
  FAR_FRONT_RIGHT = 16
} SPARK_SPATIAL_RELATION;

typedef struct STRUCT_SPARK_SYMB_PLACEMENT {
  GEN_STRING64 object;
  SPARK_PLACEMENT_RELATION relation;
} SPARK_SYMB_PLACEMENTS;

typedef struct STRUCT_SPARK_VISIBILITY_STATE {
  GEN_STRING64 agentName;
  SPARK_VISIBILITY visibility;
  SPARK_VISIBILITY_PLACEMENT visibilityPlacement;
  int pointedAt;
  SPARK_SPATIAL_RELATION spatialRelation;
} SPARK_VISIBILITY_STATE;

typedef struct STRUCT_SPARK_OBJECT_STATE {
  GEN_STRING64  name;
  unsigned long lastTimeSeen;
  unsigned long lastTimeStatic;
  int detected;
  int disappeared;
  SPARK_6D_COORD pos;
  SPARK_OBJECT_MOTION motion;
  SPARK_VISIBILITY_STATE agentVisibilityState[SPARK_MAX_AGENT_NB];
  int agentVisibilityStateNb;
  SPARK_SYMB_PLACEMENTS placements[SPARK_OBJ_STATE_MAX];
  int placementLength;
  int exists;
} SPARK_OBJECT_STATE;

//Type of situation assessment processing
typedef enum ENUM_SPARK_PROCESSING_TYPE {
  SPARK_REACH_PROC = 1,
  SPARK_LOOKAT_PROC = 2,
  SPARK_PLACEMENT_REL_PROC = 3
} SPARK_PROCESSING_TYPE;

// Type of situation assessment update
// use to force reprocessing, to print value or update some parameters values.
typedef enum ENUM_SPARK_SA_UPDATE_TYPE {
  SPARK_UPDATE_ALL = 1,
  SPARK_BLOCK_UPDATE_ALL = 2,
  SPARK_DEBLOCK_UPDATE_ALL = 3,
  SPARK_UPDATE_LOOK_AT = 4,
  SPARK_PRINT_ALL_UPDATE = 5,
  SPARK_NOPRINT_ALL_UPDATE = 6,
  SPARK_PRINT_ALL_SPARK_ORO = 7,
  SPARK_CREATE_VISIBILITY_IMAGES_ON = 8,
  SPARK_CREATE_VISIBILITY_IMAGES_OFF = 9,
  SPARK_REEXPORT_ALL_SPARK_ORO = 10,
  SPARK_ALLOW_DISAPPEAR = 11,
  SPARK_FOREBID_DISAPPEAR = 12,
  SPARK_PRINT_VIMAN_INPUT = 13,
  SPARK_NOPRINT_VIMAN_INPUT = 14,
  SPARK_SWITCH_PRINT_SA_RECOMPUTE_STATUS = 15,
  SPARK_UPDATE_DISAPEAR_ITER_MAX = 16,
  SPARK_UPDATE_REACHABILITY_DISTANCE = 17,
  SPARK_UPDATE_IS_ON_MAX_DIST = 18,
  SPARK_SWITCH_DRAW_ACTION_MONITORING_SPHERES = 19
} SPARK_SA_UPDATE_TYPE;
  
typedef struct STRUCT_SPARK_SA_UPDATE_PARAM {
  SPARK_SA_UPDATE_TYPE type;
  int intUpdatedValue;
  double doubleUpdatedValue;
} SPARK_SA_UPDATE_PARAM;

typedef struct STRUCT_SPARK_SPECIFIC_SA_PARAM {
  GEN_STRING64 agent;
  GEN_STRING64 entity1;
  GEN_STRING64 entity2;
  SPARK_PROCESSING_TYPE processingType;
} SPARK_SPECIFIC_SA_PARAM;

// Type of agent activity motion update
typedef enum ENUM_SPARK_AGENT_ACTIVITY_MOTION_TYPE {
 SPARK_AGENT_ACTIVITY_MOTION_MOVING=0,
 SPARK_AGENT_ACTIVITY_MOTION_STATIC,
 SPARK_AGENT_ACTIVITY_MOTION_HAS_MOVED,
 SPARK_AGENT_ACTIVITY_MOTION_DID_NOT_MOVE,  
 SPARK_AGENT_ACTIVITY_MOTION_STATUS_UNKNOWN, 
 SPARK_MAXI_NUM_AGENTS_ACTIVITY_MOTION_STATUS
} SPARK_AGENT_ACTIVITY_MOTION_TYPE;

// Type of agent activity motion update
typedef enum ENUM_SPARK_AGENT_ACTIVITY_MOTION_TRANSITION {
 SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_TRUE=0,
 SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_FALSE=1,
 SPARK_AGENT_ACTIVITY_ISMOVING_NO_CHANGE=2,
 SPARK_AGENT_ACTIVITY_ISMOVING_IMPOSSIBLE_TRANSITION=3
} SPARK_AGENT_ACTIVITY_MOTION_TRANSITION;


/* typedef struct STRUCT_SPARK_VISREACH_RESULT { */
/*   GEN_STRING64 result1; */
/*   GEN_STRING64 result2; */
/* } SPARK_VISREACH_RESULT; */

/* typedef enum ENUM_SPARK_REACHABILITY { */
/*   UNREACHABLE = 0, */
/*   REACHABLE = 1, */
/*   REACHABLE_WITH_COLLISION = 2, */
/*   REACHABILITY_UNKNOWN = 3 */
/* } SPARK_REACHABILITY; */

/* typedef struct STRUCT_SPARK_AGENT_REACHABILITY { */
/*   GEN_STRING128 agentName; */
/*   GEN_STRING128 objectName; */
/*   SPARK_REACHABILITY reachability; */
/* } SPARK_AGENT_REACHABILITY; */

typedef struct STRUCT_SPARK_SERVER_ADDRESS {
  GEN_STRING64 address;
  GEN_STRING64 port;
} SPARK_SERVER_ADDRESS;

/* ------------- BEGIN NEW STRUCTURES --------------- */
/* NEW STRUCTURES TO REPLACE OBJECT STATES */

/* ----- FACTS ABOUT OBJECTS ------- */
/* typedef struct  STRUCT_SPARK_PLLOCATION_COUPLE { */
/*   int plLocation; */
/*   int agentIdx; */
/* } SPARK_PLLOCATION_COUPLE; */

/* typedef struct  STRUCT_SPARK_SPLOCATION_COUPLE { */
/*   SPARK_PLACEMENT_RELATION spLocation; */
/*   int objectIdx; */
/* } SPARK_SPLOCATION_COUPLE; */

/* typedef struct  STRUCT_SPARK_VISLOCATION_COUPLE { */
/*   SPARK_VISIBILITY_PLACEMENT visLocation; */
/*   int agentIdx; */
/* } SPARK_VISLOCATION_COUPLE; */

/* typedef struct STRUCT_SPARK_OBJECT_FACTS { */
/*   int isPresent; */
/*   int plLocationNb; */
/*   int spLocationNb; */
/*   int visLocationNb; */
/*   SPARK_PLLOCATION_COUPLE plLocation[SPARK_OBJ_STATE_MAX]; */
/*   SPARK_SPLOCATION_COUPLE spLotation[SPARK_OBJ_STATE_MAX]; */
/*   SPARK_VISLOCATION_COUPLE visLocation[SPARK_OBJ_STATE_MAX]; */
/* } SPARK_OBJECT_FACTS; */

/* ----- FACTS ABOUT AGENTS --------- */
/* typedef struct STRUCT_SPARK_SEES_COUPLE { */
/*   SPARK_VISIBILITY visibility; */
/*   int objectIdx; */
/* } SPARK_SEES_COUPLE; */

/* typedef struct STRUCT_SPARK_REACHES_COUPLE { */
/*   SPARK_REACHABILITY reachability; */
/*   int objectIdx; */
/* } SPARK_REACHES_COUPLE; */

/* typedef struct STRUCT_SPARK_AGENT_FACTS { */
/*   int isPresent; */
  
/*   int handsAreMoving; */
/*   int headIsMoving; */
/*   int bodyIsMoving; */
  
/*   int seesNb; */
/*   int reachesNb; */
/*   SPARK_SEES_COUPLE sees[SPARK_OBJ_STATE_MAX]; */
/*   SPARK_REACHES_COUPLE reaches[SPARK_OBJ_STATE_MAX]; */
  
/*   int looksAt[SPARK_OBJ_STATE_MAX]; */
/*   int pointsAt[SPARK_OBJ_STATE_MAX]; */
  
/* } SPARK_AGENT_FACTS; */

/* ------------- END NEW STRUCTURES --------------- */

/** ------------- BEGIN VISUAL ATTENTION STRUCTURES --------------- */
/** This information will be used in supervisor to drive visual attention according to (among other) last time seen, level of precision, */

typedef struct STRUCT_SPARK_OBJECT_POSITION_UPDATE_MANAGEMENT_STATE {
  GEN_STRING64  objectName;
  int entityIndex;
  SPARK_3D_COORD lastPrecisePosition;
  unsigned long lastTimeSeenPrecisely;
  SPARK_3D_COORD lastRoughPosition;
  unsigned long lastTimeSeenRoughly;
  SPARK_3D_COORD lastInferredPosition;
  unsigned long lastTimeInferred;
  int hasInferrence; //Is current object position
  SPARK_INFERRENCE_TYPE inferrenceType; //what is the type of this infference
  GEN_STRING64  inferrenceObjectOrAgentPartName; //Agent, object name
  int agentPartNum; // int to discriminate betwee one Agent parts ( example hands )
  double perceptionInferrenceConflictThreshold; //threshold use to decide of perception inference conflicts classes
  double perceptionInferrenceConflictValue; // perception inference conflicts value
  SPARK_INFERRENCE_VALIDITY inferrenceValidity; // perception inference conflicts class
  unsigned long lastTimePerceptionInferrenceConflict; // last time for perception inference conflict
  double speed;
  double average_speed;
  double dist;
  double average_dist;
  double distanceRoughToPrecise;
  int detected;
  int detectedOnce;
  int disappeared;
  int isOnSpecialSupport; //Hack to get isOn for given furniture to bypass ontology.
} SPARK_OBJECT_POSITION_UPDATE_MANAGEMENT_STATE;

/** status to describe current object position measurement*/
typedef enum ENUM_SPARK_OBJECT_MEASUREMENT_TYPE {
  PRECISE_MOVING = 0,
  PRECISE_STATIC = 1,
  ROUGH_MOVING =  2,
  ROUGH_STATIC = 3,
  INFERRED_MOVING =  4,
  INFERRED_STATIC = 5,
  NO_INFO = 6
} SPARK_OBJECT_MEASUREMENT_TYPE;

/** ------------- END VISUAL ATTENTION STRUCTURES  --------------- */

/** ------------- BEGIN ACTION MONITORING STRUCTURES --------------- */
/** This information will be used in supervisor to monitor agents actions by detection the exit from or the entrance in some sphere */

/** sphere type used for initial sphere geometric dimension processing and monitoring trigger assessment*/
/** Duplicate m3d enum. Can it be avoided simply? */
typedef enum ENUM_SPARK_SPHERE_TYPE {
  SPARK_SIMPLE_ENTRY = 0, // simple sphere defined through radius and center. To monitor entrance in the sphere;
  SPARK_SIMPLE_EXIT = 1,  // simple sphere defined through radius and center. To monitor exit from the sphere.
  SPARK_THROW_IN_CONTAINER = 2, // simple sphere those radius and center is automatically defined as fitting above the container.
  SPARK_PICK_OBJECT =  3, // simple sphere those radius and center is automatically defined as around object to pick.
  SPARK_PERMANENT_STOP_MONITOR = 4  //
} SPARK_SPHERE_TYPE; // Monitor that hands are fixed at some points.

typedef struct STRUCT_SPARK_ACTION_MONITORING_SPHERE {
  int isSphereActive; // TRUE if monitor is active, FALSE otherwise.
  GEN_STRING64  agentName; // agent name whose action is monitored.
  int agentIndex; // agent index.
  GEN_STRING64  objectName; // Object name used to define sphere for certain sphereType
  int entityIndex; // entity index of the object 
  int handIndexInput; // -1 if we don't want to precise hand index > -1 otherwise.
  SPARK_3D_COORD sphereCenter; //  sphere center. 
  double sphereRadius; // sphere radius.
  double filteringTimeThreshold; // time delay to wait for monitor success condition to triger monitor success
  SPARK_SPHERE_TYPE sphereType; // what is the type of this sphere as far as creating it is concerned
  int monitorEnterInResult; // TRUE if monitor trigger for enter in spheres and FALSE otherwise.
  int monitorGetOutResult; // TRUE if monitor trigger for get out of spheres (if it was in) and FALSE otherwise.
  int handIndexResult; // Whose agent hands trigger monitor.  
  int modifIndex; // each time there is something new on this sphere the index is incremented. This is used to update poster from move3d spheres.
} SPARK_ACTION_MONITORING_SPHERE;

typedef struct STRUCT_SPARK_ACTION_MONITORING_SPHERE_INPUT_PARAMS {
  int monitorIndex; // index in the monitor array
  GEN_BOOL activateSphere; // TRUE to start monitor, FALSE to stop it. 
  GEN_STRING64 objectName; // Object name used to define sphere for certain sphereType
  int handIndexInput; // -1 if we don't want to precise hand index > -1 otherwise.
  GEN_STRING64 agentName; // agent name whose action is monitored.
  SPARK_3D_COORD sphereCenter; //  sphere center. 
  double sphereRadius; // sphere radius.
  double sphereRadiusMultiply; // Factor to increase or decrease Defaults Sphere radius 
  double sphereCenterTranslationMultiply; //Factor to increase or decrease Default translation used to get sphere center for compute.
  double filteringTimeThreshold; // time delay to wait for monitor success condition to triger monitor success
  SPARK_SPHERE_TYPE sphereType; // what is the type of this sphere as far as creating it is concerned
} SPARK_ACTION_MONITORING_SPHERE_INPUT_PARAMS;

/** ------------- END VISUAL ATTENTION STRUCTURES  --------------- */

//-------------------- POSTERS -----------------------

typedef struct STRUCT_ALL_OBJECTS_SPARK_POSITION_UPDATE_MANAGEMENT_STATE {
  SPARK_OBJECT_POSITION_UPDATE_MANAGEMENT_STATE objects[SPARK_OBJECT_POSITION_UPDATE_MANAGEMENT_STATE_OBJ_MAX];
  int length;
  unsigned long time;
} SPARK_ALL_OBJECTS_POSITION_UPDATE_MANAGEMENT_STATE;

typedef struct STRUCT_SPARK_ALL_MONITORING_SPHERES {
  SPARK_ACTION_MONITORING_SPHERE spheres[SPARK_NUM_SPHERES_MAX];
  int modifIndex;  //each time there is something new in spheres the index is incremented. This is used to update poster from move3d spheres.
} SPARK_ALL_MONITORING_SPHERES;

typedef struct STRUCT_SPARK_ALL_OBJECT_STATES {
  SPARK_OBJECT_STATE object[SPARK_OBJ_STATE_MAX];
  int length;
} SPARK_ALL_OBJECT_STATES;

//typedef struct STRUCT_SPARK_CURRENT_ENVIRONMENT {
//  GEN_STRING64 envName;
//  M3D_ROBOT robot[SPARK_MAX_THINGS_NB];
//  int robotNb;
//  int time;
//} SPARK_CURRENT_ENVIRONMENT;


typedef struct STRUCT_SPARK_CURRENT_ENVIRONMENT {
  GEN_STRING64 envName;
  M3D_ROBOT robot[SPARK_MAX_AGENT_NB];
  M3D_FREEFLYER freeflyer[SPARK_MAX_FREEFLYER_NB];
  int robotNb;
  int freeflyerNb;
  int time;
  int unused;
} SPARK_CURRENT_ENVIRONMENT;


typedef struct STRUCT_SPARK_BOUNDING_BOXES {
  GEN_STRING64 envName;
  SPARK_BOUNDING_BOX bbox[SPARK_MAX_THINGS_NB*2];
  int robotNb;
  int unused;
} SPARK_BOUNDING_BOXES;

#if defined(USE_MIGHTABILITY_MAPS)
typedef struct STRUCT_MIGHTABILITY_FLAGS {
  int MIGHTABILITY_INITIALISED;
  int SATRT_MIGHTABILITY_UPDATE;
  int STOP_MIGHTABILITY_UPDATE;
  int PAUSE_MIGHTABILITY_UPDATE;
  
}STRUCT_MIGHTABILITY_FLAGS;

typedef struct STRUCT_AROUND_OBJ_INFO_FOR_MM
{
char around_obj_name[100];
}STRUCT_AROUND_OBJ_INFO_FOR_MM;

typedef struct STRUCT_AGENT_ACTIVITY_FACTS
{
  char agent_name[100];
 char agent_motion[50];
 char body_turn[50];
 char torso_turn[50];
 char head_turn[50];

 char right_hand_status[50];
 char right_hand_config_mode[50];
 char right_hand_occup_mode[50];
 char right_hand_rest_mode_type[50];
 char right_hand_on_support[50];
 char right_hand_holding_object[50];

 char left_hand_status[50];
 char left_hand_config_mode[50];
 char left_hand_occup_mode[50];
 char left_hand_rest_mode_type[50];
 char left_hand_on_support[50];
 char left_hand_holding_object[50];
}STRUCT_AGENT_ACTIVITY_FACTS;

typedef struct STRUCT_AGENTS_ACTIVITY_FACTS
{
  STRUCT_AGENT_ACTIVITY_FACTS Agent_Facts[4];
}STRUCT_AGENTS_ACTIVITY_FACTS;

typedef struct STRUCT_SHOW_OBJECT_FACTS_FLAGS
{

//int show_by_circle;
int show_arrow;

}STRUCT_SHOW_OBJECT_FACTS_FLAGS;


#endif

#endif
